package com.cultural.heritage.controller.clothes;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.clothes.ClothesAddRequest;
import com.cultural.heritage.model.dto.clothes.ClothesQueryRequest;
import com.cultural.heritage.model.dto.clothes.ClothesUpdateRequest;
import com.cultural.heritage.model.entity.Clothes;
import com.cultural.heritage.model.vo.clothes.ClothesVO;
import com.cultural.heritage.service.clothes.ClothesService;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.utils.DecoderUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/clothes")
@Slf4j
@Tag(name = "服装租赁产品管理模块")
public class ClothesController {

    @Resource
    private ClothesService clothesService;


    @Resource
    private CommonService commonService;


    /**
     * Web端管理员添加服装租赁产品
     * @param clothesAddRequest 服装租赁产品添加请求体
     * @return 服装租赁产品id
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加服装租赁产品", description = "参数：服装租赁产品添加请求体，权限：管理员(admin, boss)，方法名：addClothes")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Long> addClothes(@RequestBody ClothesAddRequest clothesAddRequest) {
        if (clothesAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Clothes clothes = commonService.copyProperties(clothesAddRequest, Clothes.class);
        // 校验
        clothesService.validClothes(clothes, false);
        boolean save = clothesService.save(clothes);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "服装租赁产品添加失败");
        return ResultUtils.success(clothes.getId());
    }


    /**
     * Web端管理员更新服装租赁产品
     * @param clothesUpdateRequest 服装租赁产品更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新服装租赁产品", description = "参数：服装租赁产品更新请求体，权限：管理员(admin, boss)，方法名：updateClothes")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateClothes(@RequestBody ClothesUpdateRequest clothesUpdateRequest) {
        if (clothesUpdateRequest == null || clothesUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 更新服装租赁产品
        Clothes clothes = commonService.copyProperties(clothesUpdateRequest, Clothes.class);
        clothesService.validClothes(clothes, true);

        boolean result = clothesService.updateById(clothes);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装租赁产品更新失败");

        return ResultUtils.success(true);
    }


    /**
     * Web端管理员根据id删除服装租赁产品
     * @param commonRequest 服装租赁产品id
     * @return 是否删除成功
     */
    @PostMapping("/delete")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    @Operation(summary = "Web端管理员根据id删除服装租赁产品", description = "参数：服装租赁产品id，权限：管理员(admin, boss)，方法名：deleteClothes")
    public BaseResponse<Boolean> deleteClothes(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        boolean result = clothesService.removeById(commonRequest.getId());
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装租赁产品删除失败");
        return ResultUtils.success(true);
    }



    /**
     * web端用户根据id查询服装主要信息
     * @param commonRequest 服装id
     * @return  服装信息
     */
    @PostMapping("/query/id")
    @Operation(summary = "web端用户根据id查询服装主要信息", description = "参数：服装id，权限：所有人，方法名：webGetClothesById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<ClothesVO> webGetClothesById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        return ResultUtils.success(clothesService.getClothesById(commonRequest.getId()));
    }




    /**
     * 根据服装id查询服装信息
     */
    @PostMapping("/get/id")
    @Operation(summary = "小程序端用户根据id查询服装", description = "参数：服装id，权限：所有人，方法名：getClothesById")
    public BaseResponse<ClothesVO> getClothesById(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null) throw new BusinessException(ErrorCode.PARAMS_ERROR);
        return ResultUtils.success(clothesService.getClothesById(commonRequest.getId()));
    }




    /**
     * Web端管理员分页查询服装
     * @param clothesQueryRequest 查询服装请求体
     * @return 服装信息列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询服装", description = "参数：服装查询请求体，排序字段 sortField: price inventory id，权限：管理员(admin, boss)，方法名：listClothesByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<ClothesVO>> listClothesByPage(@RequestBody ClothesQueryRequest clothesQueryRequest) {
        long current = clothesQueryRequest.getCurrent();
        long pageSize = clothesQueryRequest.getPageSize();
        QueryWrapper<Clothes> clothesQueryWrapper = clothesService.getQueryWrapper(clothesQueryRequest);
        Page<Clothes> page = clothesService.page(new Page<>(current, pageSize), clothesQueryWrapper);
        List<Clothes> clothesList = page.getRecords();
        // 处理富文本字段
        for (Clothes clothes : clothesList) {
            if (StringUtils.isNotBlank(clothes.getRichText())) clothes.setRichText(DecoderUtils.decodeText(clothes.getRichText()));
        }
        List<ClothesVO> clothesVOS = commonService.convertList(clothesList, ClothesVO.class);
        Page<ClothesVO> clothesPageVOPage = new Page<>();
        clothesPageVOPage.setRecords(clothesVOS);
        clothesPageVOPage.setTotal(page.getTotal());
        clothesPageVOPage.setSize(page.getSize());
        clothesPageVOPage.setCurrent(page.getCurrent());
        clothesPageVOPage.setPages(page.getPages());

        return ResultUtils.success(clothesPageVOPage);
    }






    /**
     * Web端管理员批量删除服装
     * @param commonDelBatchRequest 批量删除请求体
     * @return 是否批量删除成功
     */
    @PostMapping("/delBatch")
    @Operation(summary = "Web端管理员批量删除服装", description = "参数：服装批量删除请求体，权限：管理员(admin, boss)，方法名：delBatchClothes")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBatchClothes(@RequestBody CommonBatchRequest commonDelBatchRequest) {
        if (commonDelBatchRequest == null || CollectionUtils.isEmpty(commonDelBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Long> idList = commonDelBatchRequest.getIdList();
        // 批量删除服装
        boolean result = clothesService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服装批量删除失败");

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员上(下)架服装
     * @param commonRequest 服装id
     * @return 是否更新成功
     */
    @PostMapping("/shelves")
    @Operation(summary = "Web端管理员上(下)架服装", description = "参数：服装id，权限：管理员(admin, boss)，方法名：updateClothesShelvesStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateClothesShelvesStatus(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取当前服装的上(下)架状态
        Long id = commonRequest.getId();
        Clothes clothes = clothesService.getById(id);
        ThrowUtils.throwIf(clothes == null, ErrorCode.OPERATION_ERROR, "服装不存在");
        Integer status = clothes.getIsShelves() == 0 ? 1 : 0;
        UpdateWrapper<Clothes> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("isShelves", status);
        boolean update = clothesService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "上架状态更新失败");
        return ResultUtils.success(true);
    }




}
