package com.cultural.heritage.controller.order;


import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.advanceOrder.AdvanceOrderAddRequest;
import com.cultural.heritage.model.dto.advanceOrder.AdvanceOrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderStatusUpdateRequest;
import com.cultural.heritage.model.dto.snapshot.ContactsSnapshot;
import com.cultural.heritage.model.dto.snapshot.PhotoProductsSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.model.vo.advanceorder.AdvanceOrderVO;
import com.cultural.heritage.service.address.ContactsService;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.service.book.BookingDateService;
import com.cultural.heritage.service.book.BookingTimeService;
import com.cultural.heritage.service.book.PhotoProductsService;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.user.UserService;
import com.cultural.heritage.utils.OrderNumberUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/advanceOrder")
@Slf4j
@Tag(name = "写真预约订单管理模块")
public class AdvanceOrderController {


    @Resource
    private AdvanceOrderService advanceOrderService;


    @Resource
    private UserService userService;


    @Resource
    private ContactsService contactsService;


    @Resource
    private PhotoProductsService photoProductsService;


    @Resource
    private CommonService commonService;


    @Resource
    private BookingDateService bookingDateService;


    @Resource
    private BookingTimeService bookingTimeService;




    /**
     * 小程序端用户创建写真预约订单
     * @param addAdvanceOrderRequest    写真预约订单添加请求体
     * @return  订单id
     */
    @PostMapping("/add")
    @Operation(summary = "小程序端用户创建写真预约订单", description = "参数：写真预约订单添加请求体, 权限：所有人，方法名：addAdvanceOrder")
    public BaseResponse<Long> addAdvanceOrder(@RequestBody AdvanceOrderAddRequest addAdvanceOrderRequest, HttpServletRequest request) {
        if (addAdvanceOrderRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取用户id
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();

        // 获取写真产品信息
        Long photoProductId = addAdvanceOrderRequest.getPhotoProductId();
        PhotoProducts photoProducts = photoProductsService.getById(photoProductId);
        ThrowUtils.throwIf(photoProducts == null || photoProducts.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "当前写真产品不存在或者已下架");
        PhotoProductsSnapshot photoProductsSnapshot = new PhotoProductsSnapshot();
        BeanUtils.copyProperties(photoProducts, photoProductsSnapshot);

        // 校验当前订单的预约日期和预约时间是否存在
        Long bookingDateId = addAdvanceOrderRequest.getBookingDateId();
        Long bookingTimeId = addAdvanceOrderRequest.getBookingTimeId();
        BookingDate bookingDate = bookingDateService.getById(bookingDateId);
        ThrowUtils.throwIf(bookingDate == null || bookingDate.getIsAvailable() == 0, ErrorCode.OPERATION_ERROR, "当前预约日期不存在或者已不可预约");
        BookingTime bookingTime = bookingTimeService.getById(bookingTimeId);
        ThrowUtils.throwIf(bookingTime == null, ErrorCode.OPERATION_ERROR, "当前预约时间不存在");

        // 校验当前订单的拍摄场景是否存在
        String shotScene = photoProducts.getShotScene();
        Integer isIndoors = addAdvanceOrderRequest.getIsIndoors();
        ThrowUtils.throwIf(shotScene.equals("室内") && isIndoors == 0 || shotScene.equals("室外") && isIndoors == 1, ErrorCode.OPERATION_ERROR, "拍摄场景不存在");

        // 校验当前订单预约时间是否被预约过或者已过期
        String specificDate = bookingDate.getSpecificDate();
        String timePoint = bookingTime.getTimePoint();
        advanceOrderService.validBookingTime(specificDate, timePoint);
        // 获取联系人信息
        Long contactsId = addAdvanceOrderRequest.getContactsId();
        Contacts contacts = contactsService.getById(contactsId);
        ThrowUtils.throwIf(contacts == null, ErrorCode.OPERATION_ERROR, "联系人信息不存在");
        ContactsSnapshot contactsSnapshot = new ContactsSnapshot();
        BeanUtils.copyProperties(contacts, contactsSnapshot);
        // 计算订单总金额
        BigDecimal totalAmount = photoProducts.getPrice();
        // 生成订单编号
        String orderNumber = OrderNumberUtils.generateOrderId();
        // 封装写真预约订单
        AdvanceOrder advanceOrder = new AdvanceOrder();
        BeanUtils.copyProperties(addAdvanceOrderRequest, advanceOrder);
        advanceOrder.setUserId(userId);
        advanceOrder.setSpecificDate(specificDate);
        advanceOrder.setTimePoint(timePoint);
        advanceOrder.setContactsSnapshot(contactsSnapshot);
        advanceOrder.setPhotoProductsSnapshot(photoProductsSnapshot);
        advanceOrder.setTotalAmount(totalAmount);
        advanceOrder.setOrderNumber(orderNumber);
        advanceOrder.setOrderStatus(OrderStatusConstant.PENDING_PAYMENT);
        // 校验
        advanceOrderService.validAdvanceOrder(advanceOrder);
        boolean result = advanceOrderService.save(advanceOrder);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "订单创建失败");
        // 向消息队列中发送订单创建的消息
        advanceOrderService.sendAdvanceOrderMessage(advanceOrder.getId());
        return ResultUtils.success(advanceOrder.getId());
    }





    /**
     * 小程序端用户取消写真预约订单
     * @param commonRequest     订单id
     * @return   是否取消成功
     */
    @PostMapping ("/cancel/id")
    @Operation(summary = "小程序端用户取消写真预约订单", description = "参数：订单id，权限：所有人，方法名：cancelOrderById")
    public BaseResponse<Boolean> cancelOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        AdvanceOrder advanceOrder = advanceOrderService.getById(id);
        ThrowUtils.throwIf(advanceOrder == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        ThrowUtils.throwIf(!advanceOrder.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.SYSTEM_ERROR, "订单状态错误");
        advanceOrder.setOrderStatus(OrderStatusConstant.TRANSACTION_CLOSED);
        boolean update = advanceOrderService.updateById(advanceOrder);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态更新失败");

        return ResultUtils.success(true);
    }







    /**
     * Web端管理员查询写真预约订单
     * @param advanceOrderQueryRequest  写真预约订单查询请求体
     * @return  写真预约订单列表
     */
    @PostMapping("/list")
    @Operation(summary = "Web端管理员查询写真预约订单", description = "参数：写真预约订单查询请求体, 权限：管理员(admin, boss)，方法名：listAdvanceOrderByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<AdvanceOrderVO>> listAdvanceOrderByPage(@RequestBody AdvanceOrderQueryRequest advanceOrderQueryRequest) {
        if (advanceOrderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = advanceOrderQueryRequest.getCurrent();
        long pageSize = advanceOrderQueryRequest.getPageSize();
        QueryWrapper<AdvanceOrder> queryWrapper = advanceOrderService.getQueryWrapper(advanceOrderQueryRequest);
        Page<AdvanceOrder> page = advanceOrderService.page(new Page<>(current, pageSize), queryWrapper);
        // 封装成写真预约订单VO
        List<AdvanceOrder> records = page.getRecords();
        List<AdvanceOrderVO> advanceOrderVOS = commonService.convertList(records, AdvanceOrderVO.class);
        // TODO
//        for (AdvanceOrderVO advanceOrderVO : advanceOrderVOS) {
//            PhotoProductsSnapshot photoProductsSnapshot = advanceOrderVO.getPhotoProductsSnapshot();
//            photoProductsSnapshot.setIntroImg(FileConstant.SERVER_HOST + photoProductsSnapshot.getIntroImg());
//            advanceOrderVO.setPhotoProductsSnapshot(photoProductsSnapshot);
//        }
        Page<AdvanceOrderVO> voPage = new Page<>();
        voPage.setRecords(advanceOrderVOS);
        voPage.setTotal(page.getTotal());
        voPage.setPages(page.getSize());
        voPage.setCurrent(page.getCurrent());
        voPage.setSize(page.getSize());
        return ResultUtils.success(voPage);
    }




    /**
     * 小程序端用户查询写真预约订单
     * @return  当前用户的订单列表
     */
    @PostMapping("/list/my")
    @Operation(summary = "小程序端用户查询写真预约订单", description = "参数：无, 权限：所有人，方法名：listMyAdvanceOrder")
    public BaseResponse<List<AdvanceOrderVO>> listMyAdvanceOrder(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();
        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", userId);
        List<AdvanceOrder> advanceOrderList = advanceOrderService.list(queryWrapper);
        // TODO
//        for (AdvanceOrder advanceOrder : advanceOrderList) {
//            PhotoProductsSnapshot photoProductsSnapshot = advanceOrder.getPhotoProductsSnapshot();
//            photoProductsSnapshot.setIntroImg(FileConstant.SERVER_HOST + photoProductsSnapshot.getIntroImg());
//            advanceOrder.setPhotoProductsSnapshot(photoProductsSnapshot);
//        }
        // 封装成写真预约订单VO
        List<AdvanceOrderVO> advanceOrderVOS = commonService.convertList(advanceOrderList, AdvanceOrderVO.class);
        Collections.reverse(advanceOrderVOS);
        return ResultUtils.success(advanceOrderVOS);
    }



    /**
     * 小程序端用户根据id删除写真预约订单
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "小程序端用户根据id删除写真预约订单", description = "参数：无, 权限：所有人，方法名：delAdvanceOrderById")
    public BaseResponse<Boolean> delAdvanceOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = commonRequest.getId();
        AdvanceOrder advanceOrder = advanceOrderService.getById(id);
        ThrowUtils.throwIf(advanceOrder == null || !advanceOrder.getOrderStatus().equals(OrderStatusConstant.TRANSACTION_CLOSED), ErrorCode.OPERATION_ERROR, "订单不存在或状态错误");
        boolean result = advanceOrderService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "订单删除失败");
        return ResultUtils.success(true);
    }




    /**
     * 小程序端校验写真产品立即预约
     * @param commonRequest     写真产品id
     * @return   产品是否通过校验
     */
    @PostMapping ("/check/photoProducts")
    @Operation(summary = "小程序端校验写真产品立即预约", description = "参数：写真产品id，权限：所有人，方法名：cancelOrderById")
    public BaseResponse<Boolean> checkPhotoProducts(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        PhotoProducts photoProducts = photoProductsService.getById(id);
        ThrowUtils.throwIf(photoProducts == null || photoProducts.getIsShelves() == 0, ErrorCode.OPERATION_ERROR, "当前写真产品不存在或者已下架");
        // 判断当前写真产品是否存在预约时间段

        // 获取所有写真预约类订单
        Map<String, Integer> isOccupiedMap = new HashMap<>();
        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderStatus", OrderStatusConstant.PENDING_PAYMENT).or().eq("orderStatus", OrderStatusConstant.PENDING_SHIPMENT);
        List<AdvanceOrder> advanceOrderList = advanceOrderService.list(queryWrapper);
        for (AdvanceOrder advanceOrder : advanceOrderList) {
            String specificDate = advanceOrder.getSpecificDate();
            String timePoint = advanceOrder.getTimePoint();
            isOccupiedMap.put(specificDate + "&" + timePoint, 1);
        }

        // 获取预约日期表
        List<BookingDate> bookingDateList = commonService.findByFieldEqTargetField("photoProductId", id, bookingDateService);
        // 获取预约时间表
        List<BookingTime> bookingTimeList = commonService.findByFieldInTargetField(bookingDateList, bookingTimeService, BookingDate::getId, "bookingDateId");

        boolean isExistAvailable = false;
        for (BookingDate bookingDate : bookingDateList) {
            Long bookingDateId = bookingDate.getId();
            // 过滤掉没有空闲的预约日期
            if (bookingDate.getIsAvailable() == 0) continue;
            // 过滤掉已过期的预约日期
            String specificDate = bookingDate.getSpecificDate();
            String today = DateUtil.today();
            int result = DateUtil.compare(DateUtil.parse(specificDate), DateUtil.parse(today));
            if (result < 0) continue;
            // 如果预约日期相同，过滤掉预约时间以及被占用的时间
            List<BookingTime> bookingTimes = bookingTimeList.stream().filter(bookingTime -> bookingTime.getBookingDateId().equals(bookingDateId)).toList();
            if (result == 0) {
                for (BookingTime bookingTime : bookingTimes) {
                    String point = bookingTime.getTimePoint();
                    String timePoint = point + ":00";
                    DateTime targetTime = DateUtil.parse(timePoint, "HH:mm:ss");
                    DateTime now = DateUtil.date();
                    DateTime currentTime = DateUtil.parse(now.toString("HH:mm:ss"), "HH:mm:ss");
                    isExistAvailable = currentTime.compareTo(targetTime) < 0 && isOccupiedMap.get(specificDate + "&" + point) == null;
                    if (isExistAvailable) break;
                }
            } else {
                for (BookingTime bookingTime : bookingTimes) {
                    String timePoint = bookingTime.getTimePoint();
                    isExistAvailable = isOccupiedMap.get(specificDate + "&" + timePoint) == null;
                    if (isExistAvailable) break;
                }
            }
            if (isExistAvailable) break;
        }
        ThrowUtils.throwIf(!isExistAvailable, ErrorCode.OPERATION_ERROR, "当前写真产品的所有时段已约满");
        return ResultUtils.success(true);
    }



    /**
     * Web端管理员更新写真预约订单状态
     * @param orderStatusUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update/orderStatus")
    @Operation(summary = "Web端管理员更新写真预约订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateOrderStatus(@RequestBody OrderStatusUpdateRequest orderStatusUpdateRequest) {
        if (orderStatusUpdateRequest == null || orderStatusUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderStatusUpdateRequest.getId();
        String orderStatus = orderStatusUpdateRequest.getOrderStatus();
        ThrowUtils.throwIf(StringUtils.isBlank(orderStatus), ErrorCode.OPERATION_ERROR, "订单状态参数为空");
        UpdateWrapper<AdvanceOrder> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = advanceOrderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "订单状态不存在或订单状态更新失败");
        return ResultUtils.success(true);
    }




}
